
//  Foreach loop for NSIndexSet
//  Copyright © 2011  Michel Fortin
//  <http://michelf.com/>
//
// This file is published under the terms of the Boost Software License 1.0.
//
// Boost Software License - Version 1.0 - August 17th, 2003
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
//
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

#import <Foundation/Foundation.h>

/*
MFIndexSetForeach is a macros to easily iterate over all indices in an index set.
Use like this:

	NSUInteger count = 0;
	MFIndexSetForeach(index, indexSet)
	{
	    if (index % 2) ++count;
	    if (index % 33) break;
	}
*/
#define MFIndexSetForeach(indexVar, set) \
	MFIndexSetForeachWithBufferLength(indexVar, set, 16)

/*
MFIndexSetForeachWithBufferLength lets you choose a different length for the buffer
holding temporary indexes, changing memory requirements on the stack and the number
of calls to `getIndexes:maxCount:inIndexRange:`.
*/
#define MFIndexSetForeachWithBufferLength(indexVar, set, bufferLength) \
	_MFIndexSetForeachWithBufferLengthLine(indexVar, set, bufferLength, __LINE__)


// Implementation details:

#define _MFIndexSetForeachConcat(a, b) a##b

#define _MFIndexSetForeachWithBufferLengthLine(indexVar, set, bufferLength, LINE) \
	NSRange _MFIndexSetForeachConcat(_mf_indexRange,LINE) = NSMakeRange(0, NSUIntegerMax); \
	for (NSUInteger indexVar, \
		 _MFIndexSetForeachConcat(_mf_bufferIndex,LINE) = bufferLength-1, \
		 _MFIndexSetForeachConcat(_mf_indexCount,LINE) = 0, \
	     _MFIndexSetForeachConcat(_mf_indexBuffer,LINE)[bufferLength]; \
	     _MFIndexSetForeachNextIndex( \
			&_MFIndexSetForeachConcat(_mf_bufferIndex,LINE), \
			&_MFIndexSetForeachConcat(_mf_indexCount,LINE), \
			set, \
			_MFIndexSetForeachConcat(_mf_indexBuffer,LINE), \
			bufferLength, \
			&_MFIndexSetForeachConcat(_mf_indexRange,LINE), &indexVar); \
		 ) \

NS_INLINE BOOL _MFIndexSetForeachNextIndex(NSUInteger *bufferIndex, NSUInteger *indexCount, NSIndexSet *set, NSUInteger indexBuffer[], NSUInteger bufferLength, NSRange *indexRange, NSUInteger * indexVar)
{
	if (++*bufferIndex >= bufferLength)
	{
		*bufferIndex = 0;
		*indexCount = [set getIndexes:indexBuffer maxCount:bufferLength inIndexRange:indexRange];
	}
	*indexVar = indexBuffer[*bufferIndex];
	return *bufferIndex < *indexCount;
}

